package com.siracusa.soa.esb.util.jersey;

import java.util.Map;

import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.core.util.MultivaluedMapImpl;
import com.sun.jersey.client.apache.ApacheHttpClient;
import com.sun.jersey.client.apache.ApacheHttpClientHandler;

import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.MultiThreadedHttpConnectionManager;

public class JerseyClientSingleton {
	static private final int MAX_PER_HOST_CONNECTIONS 	= 100;
	static private final int MAX_CONNECTIONS 			= 200;
	static private final int CONNECT_TIMEOUT_MS 		= 20000;
	static private final int READ_TIMEOUT_MS 			= 20000;

	static private Client 	_client=null;

	protected JerseyClientSingleton() {}

	static public synchronized void init(int connectTimeout, int readTimeout, int maxPerHostConnections, int maxConnections) {
		if (_client == null) {
			MultiThreadedHttpConnectionManager m = new MultiThreadedHttpConnectionManager();
			m.getParams().setDefaultMaxConnectionsPerHost(maxPerHostConnections);
			m.getParams().setMaxTotalConnections(maxConnections);
			_client = new ApacheHttpClient(new ApacheHttpClientHandler(new HttpClient(m)));
			_client.setFollowRedirects(true);
			_client.setConnectTimeout(connectTimeout);
            _client.setReadTimeout(readTimeout);
		}
	}
	static public synchronized void init() {
		JerseyClientSingleton.init(CONNECT_TIMEOUT_MS, READ_TIMEOUT_MS, MAX_PER_HOST_CONNECTIONS, MAX_CONNECTIONS);
	}

	static public ClientResponse postXML(String url, String resource, String request) {
		WebResource r = _client.resource(url);
		return r.path(resource).type(MediaType.APPLICATION_XML).post(ClientResponse.class, request);
	}

	static public ClientResponse postText(String url, String resource, String request) {
		WebResource r = _client.resource(url);
		return r.path(resource).type(MediaType.TEXT_PLAIN).post(ClientResponse.class, request);
	}

	static public ClientResponse post(String url, MediaType mediaType, String resource, String request) {
		WebResource r = _client.resource(url);
		return r.path(resource).type(mediaType).post(ClientResponse.class, request);
	}

	static public ClientResponse get(String url, MultivaluedMap<String, String> params, String resource, String request) {
		WebResource r = _client.resource(url);

		return r.path(resource).
		        queryParams(params).
		        get(ClientResponse.class);
	}

	static public ClientResponse get(String url, Map<String, String[]> requestParams, String resource, String request) {
		WebResource r = _client.resource(url);

		MultivaluedMap<String, String> params = new MultivaluedMapImpl();
        for (Map.Entry<String, String[]> p: requestParams.entrySet()) {
			String key = p.getKey();
			String[] vals = p.getValue();
			String aggregate = "";
			for (String val: vals) {
			    aggregate+=val;
			}
			params.add(key, aggregate);
         }

		return r.path(resource).
		        queryParams(params).
		        get(ClientResponse.class);
	}
}
